// Commodore PET-2001 Clone
// PS/2 keyboard and RS-232 interface
// "Matrix encoder" (U1) firmware
// Glen Kleinschmidt
// www.glensstuff.com
// Nov. 2018
// V1.0

#include <16f874A.h>
#use delay(clock=16000000)
#use standard_io(A)
#use standard_io(B)
#use standard_io(E)
#use fast_io(C)
#use fast_io(D)
#FUSES HS,NOWDT,PUT,NOPROTECT,NOLVP,NOCPD,NOWRT,BROWNOUT

int         key_code          = 0;
int1        extended_key      = 0;
int1        shift             = 0;
int         scanline          = 0;
int         M1                = 0;
int         M2                = 0;
int16       key_scan_duration = 600;
int16       counter           = key_scan_duration;
                          
                                                          
void key_code_input()
{      

   // This routine collects the current keyboard character from the serial communicator (U2)

   key_code=0;
   
   if(input(pin_A0))             // Read 7-bit key code
   bit_set(key_code,0);

   if(input(pin_A1))
   bit_set(key_code,1);
   
   if(input(pin_A2))
   bit_set(key_code,2);
   
   if(input(pin_A3))
   bit_set(key_code,3);
   
   if(input(pin_A4))
   bit_set(key_code,4);
   
   if(input(pin_A5))
   bit_set(key_code,5);
   
   if(input(pin_E0))
   bit_set(key_code,6);
     
   if(input(pin_E1))             // check EXTENDED status
   extended_key=1;
   else
   extended_key=0;
   
   if(input(pin_E2))             // Check SHIFT status
   shift=1;
   else
   shift=0;  
   
}

void MAIN()
{
  
   set_tris_c(0);                // Port C = output
   set_tris_D(255);              // Port D = input
   output_C(255);                // PIA#1 PB0 - PB7 initially set high (inputs are active low)
   delay_ms(3000);               // Wait for serial communicator (U2) to boot.
     
   loop:
     
   if(!input(pin_B0))            // Skip key-code input read if sender busy updating 
   key_code_input();
   
   // scanline_in(_ex) = the 1-of-10 keyboard matrix line select as decoded from PIA#1 PA0 - PA3
   // key0 = 255
   // key1 = 254
   // key2 = 253
   // key3 = 251
   // key4 = 247
   // key5 = 239
   // key6 = 223
   // key7 = 191
   // key8 = 63
   // key9 = 127
   
   // scanline_out(_ex) = the 1-of-8 keyboard matrix column outouts as applied to PIA#1 PB0 - PB7             
   // These PIA#1 inputs ("key in", KIN0 - KIN7) are active low.
   // PB0 = 254
   // PB1 = 253
   // PB2 = 251
   // PB3 = 247
   // PB4 = 239
   // PB5 = 223
   // PB6 = 191
   // PB7 = 127
   
   //These variable arrays map the PS/2-standard keyboard character codes to the Commodore keyboard 
   //matrix columns and lines inputs and outputs. In/out array pair "_ex" are for PS/2 extended key codes.
   //A value of 0 in scan_line_out(_ex) identifies an unused/invalid character.

   int const scanline_in    [128] ={255,255,255,255,255,255,255,127,255,255,
                                    255,255,255,254,254,255,255,127, 63,255,
                                    127,253,255,255,255,255,223,239,247,251,
                                     63,255,255,223,191,247,253,254,255,255,
                                    255,127,191,239,253,251,255,255,255,191,
                                    223,239,247,251,253,255,255,255,223,247,
                                    253,255,239,255,255,127,239,251,253,254,
                                    255,255,255, 63,191,247,223,251,255,255,
                                    255,255,191,255,127,127,255,255,255, 63,
                                    223, 63,255,254,255,255,255,255,255,255,
                                    255,255,254,255,255,223,255,247,253,255,
                                    255,255, 63,127,191,239,247,251,255,255,
                                    127,191,223, 63,239,253,255,255};                
               
   int const scanline_out   [128] ={  0,  0,  0,  0,  0,  0,  0,127,  0,  0,
                                      0,  0,  0,251,254,  0,  0,239,254,  0,
                                    254,254,254,  0,  0,  0,254,254,254,254,
                                    253,  0,  0,253,254,253,253,253,253,  0,
                                      0,251,253,253,251,253,251,  0,  0,251,
                                    251,251,251,251,223,  0,  0,  0,247,247,
                                    247,247,239,  0,  0,247,247,247,239,239,
                                    239,  0,  0,239,239,239,239,239,223,  0,
                                      0,  0,247,  0,253,127,  0,  0,  0,223,
                                    223,251,  0,247,  0,  0,  0,  0,  0,  0,
                                      0,  0,127,  0,  0,191,  0,191,191,  0,
                                      0,  0,191,191,191,191,127,191,  0,  0,
                                    191,127,127,127,127,127,  0,  0};
                                     
   int const scanline_in_ex [128] ={255,255,255,255,255,255,255,255,255,255,
                                    255,255,255,255,255,255,255,127,255,255,
                                    127,255,255,255,255,255,255,255,255,255,
                                    255,255,255,255,255,255,255,255,255,255,
                                    255,255,255,255,255,255,255,255,255,255,
                                    255,255,255,255,255,255,255,255,255,255,
                                    255,255,255,255,255,255,255,255,255,255,
                                    255,255,255,255,251,255,255,255,255,255,
                                    255,255,255,255,255,255,255,255,255,255,
                                    223,255,255,255,255,255,255,255,255,255,
                                    255,255,255,255,255,255,255,255,255,255,
                                    255,255,255,255,254,255,255,254,255,255,
                                    255,255,255,255,255,255,255,255};                                                                    
      
   int const scanline_out_ex[128] ={  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                                      0,  0,  0,  0,  0,  0,  0,239,  0,  0,
                                    254,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                                      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                                      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                                      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                                      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                                      0,  0,  0,  0,127,  0,  0,  0,  0,  0,
                                      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                                    223,  0,  0,  0,  0,  0,  0,  0,  0,  0,
                                      0,  0,  0,  0,  0,  0,  0,  0,191,  0,
                                      0,  0,  0,  0,191,  0,127,  0,  0,  0,
                                      0,  0,  0,  0,  0,  0,  0,  0};
     
   if(extended_key==0)
   {
      M1 =  scanline_in[key_code];
      M2 =  scanline_out[key_code];
   }
   else
   {
      M1 =  scanline_in_ex[key_code];
      M2 =  scanline_out_ex[key_code];
   }
      
   if(M2==0)                                    // Ignore invalid character
   {
      output_C(255);
      goto loop;
   }
      
   if(shift==1 && M1==63)
   bit_clear(M2,0);
           
   while(!input(pin_D7));                       // Wait for keyboard matrix scan interval
      
   loop2:   
      
      scanline = input_D();
               
      if(scanline == M1)
      {
         output_C(M2);
      }
      else
      {
         if(scanline == 63)                      // Assert shift key if shift=1
         {
         output_C(255-shift);
         }
         else
         {         
         output_C(255);
         }
      }
                  
      counter--;
      
      if(counter==0)
      {
         counter=key_scan_duration;
         goto loop;
      }
            
   goto loop2;  
       
}

